set names utf8;

drop table if exists `service_traff_stats_cache`;

create index storetime ON service_traff_stats(storetime);

DROP TABLE IF EXISTS `service_traff_stats_day`;
CREATE TABLE `service_traff_stats_day` (
		  `id` int(11) unsigned NOT NULL auto_increment,
		  `storetime` timestamp NOT NULL default CURRENT_TIMESTAMP,
		  `bytes` bigint(8) unsigned NOT NULL default '0',
		  `userid` int(10) unsigned NOT NULL,
		  `filterid` int(10) unsigned NOT NULL,
		  `in_traff` tinyint(1) NOT NULL default '0',
		  `cost` double unsigned NOT NULL,
		  PRIMARY KEY  (`id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;


DROP TABLE IF EXISTS `service_traff_stats_hour`;
CREATE TABLE `service_traff_stats_hour` (
		  `id` int(11) unsigned NOT NULL auto_increment,
		  `storetime` timestamp NOT NULL default CURRENT_TIMESTAMP,
		  `bytes` bigint(8) unsigned NOT NULL default '0',
		  `userid` int(10) unsigned NOT NULL,
		  `filterid` int(10) unsigned NOT NULL,
		  `in_traff` tinyint(1) NOT NULL default '0',
		  `cost` double unsigned NOT NULL,
		  PRIMARY KEY  (`id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;


DROP TABLE IF EXISTS `service_traff_stats_month`;
CREATE TABLE `service_traff_stats_month` (
		  `id` int(11) unsigned NOT NULL auto_increment,
		  `storetime` timestamp NOT NULL default CURRENT_TIMESTAMP,
		  `bytes` bigint(8) unsigned NOT NULL default '0',
		  `userid` int(10) unsigned NOT NULL,
		  `filterid` int(10) unsigned NOT NULL,
		  `in_traff` tinyint(1) NOT NULL default '0',
		  `cost` double unsigned NOT NULL,
		  PRIMARY KEY  (`id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;

DELIMITER ;;
DROP PROCEDURE IF EXISTS `cleanstats`;;
CREATE PROCEDURE `cleanstats`(IN startdate CHAR(20), IN enddate CHAR(20))
BEGIN
               DELETE FROM service_traff_stats WHERE storetime BETWEEN startdate AND enddate;
END ;;

DROP PROCEDURE IF EXISTS `sumstats_hour`;;
CREATE PROCEDURE `sumstats_hour`(IN i_date CHAR(20))
BEGIN
	DECLARE userid_int, filterid_int, b INT;
	DECLARE sum_bytes BIGINT;
	DECLARE sum_cost DOUBLE;
	DECLARE in_traff_int tinyint(1);
	DECLARE cur_1 CURSOR FOR SELECT sum(bytes), sum(cost), userid, filterid, in_traff FROM service_traff_stats WHERE storetime BETWEEN i_date AND DATE_ADD(i_date, INTERVAL 1 hour) GROUP BY userid, filterid, in_traff;
	DECLARE CONTINUE HANDLER FOR NOT FOUND
		SET b=1;
	DELETE FROM service_traff_stats_hour WHERE storetime = i_date;
	OPEN cur_1;
	WHILE NOT b <=> 1 DO
		FETCH cur_1 INTO sum_bytes, sum_cost, userid_int, filterid_int, in_traff_int;
		IF NOT b <=> 1 THEN
			INSERT INTO service_traff_stats_hour ()VALUES (NULL, i_date, sum_bytes, userid_int, filterid_int, in_traff_int, sum_cost);
		END IF;
	END WHILE;
    CLOSE cur_1;
END;;



DROP PROCEDURE IF EXISTS `sumstats_day`;;
CREATE PROCEDURE `sumstats_day`(IN i_date CHAR(20))
BEGIN
	DECLARE userid_int, filterid_int, b INT;
	DECLARE sum_bytes BIGINT;
	DECLARE sum_cost DOUBLE;
	DECLARE in_traff_int tinyint(1);
	DECLARE cur_1 CURSOR FOR SELECT sum(bytes), sum(cost), userid, filterid, in_traff FROM service_traff_stats WHERE storetime BETWEEN i_date AND DATE_ADD(i_date, INTERVAL 1 day) GROUP BY userid, filterid, in_traff;
	DECLARE CONTINUE HANDLER FOR NOT FOUND
		SET b=1;
	DELETE FROM service_traff_stats_day WHERE storetime = i_date;
	OPEN cur_1;
	WHILE NOT b <=> 1 DO
		FETCH cur_1 INTO sum_bytes, sum_cost, userid_int, filterid_int, in_traff_int;
		IF NOT b <=> 1 THEN
			INSERT INTO service_traff_stats_day ()VALUES (NULL, i_date, sum_bytes, userid_int, filterid_int, in_traff_int, sum_cost);
		END IF;
	END WHILE;
    CLOSE cur_1;
END;;

DROP PROCEDURE IF EXISTS `sumstats_month`;;
CREATE PROCEDURE `sumstats_month`(IN i_date CHAR(20))
BEGIN
	DECLARE userid_int, filterid_int, b INT;
	DECLARE sum_bytes BIGINT;
	DECLARE sum_cost DOUBLE;
	DECLARE in_traff_int tinyint(1);
	DECLARE cur_1 CURSOR FOR SELECT sum(bytes), sum(cost), userid, filterid, in_traff FROM service_traff_stats WHERE storetime BETWEEN i_date AND DATE_ADD(i_date, INTERVAL 1 month) GROUP BY userid, filterid, in_traff;
	DECLARE CONTINUE HANDLER FOR NOT FOUND
		SET b=1;
	DELETE FROM service_traff_stats_month WHERE storetime = i_date;
	OPEN cur_1;
	WHILE NOT b <=> 1 DO
		FETCH cur_1 INTO sum_bytes, sum_cost, userid_int, filterid_int, in_traff_int;
		IF NOT b <=> 1 THEN
			INSERT INTO service_traff_stats_month ()VALUES (NULL, i_date, sum_bytes, userid_int, filterid_int, in_traff_int, sum_cost);
		END IF;
	END WHILE;
    CLOSE cur_1;
END;;


DROP PROCEDURE IF EXISTS `writeoff_day` ;;
CREATE PROCEDURE `writeoff_day`()
BEGIN
	DECLARE tariff_id, service_id, delta, b INT;
	DECLARE tariff_perday, fee_perday DOUBLE;
	DECLARE tariff_last_writeoff_day, fee_last_writeoff_day TIMESTAMP;
	DECLARE cur_t CURSOR FOR SELECT id, perday, last_writeoff_day FROM tariffs;
	DECLARE cur_fee_services CURSOR FOR SELECT tariffid, serviceid, perday, last_writeoff_day FROM fee_by_services;
	DECLARE CONTINUE HANDLER FOR NOT FOUND
		SET b=1;
	OPEN cur_t;
	WHILE NOT b <=> 1 DO
		FETCH cur_t INTO tariff_id, tariff_perday, tariff_last_writeoff_day;
		IF NOT b <=> 1 THEN
			IF tariff_last_writeoff_day = '0000-00-00 00:00:00' THEN
				SET delta = 1;
			ELSE
				SET delta = DATEDIFF(now(),tariff_last_writeoff_day);
			END IF;
			
			IF delta > 0 THEN
				UPDATE users SET balance=balance-delta*tariff_perday WHERE tariffid=tariff_id;
				UPDATE tariffs SET last_writeoff_day=now() WHERE id=tariff_id;
			END IF;
		END IF;
	END WHILE;
	CLOSE cur_t;
-- Writeoff services fee
	SET b = 0;
	OPEN cur_fee_services;
	WHILE b != 1 DO
		FETCH cur_fee_services INTO tariff_id, service_id, fee_perday, fee_last_writeoff_day;
		IF NOT b <=> 1 THEN
			IF fee_last_writeoff_day = '0000-00-00 00:00:00' THEN
				SET delta = 1;
			ELSE
				SET delta = DATEDIFF(now(), fee_last_writeoff_day);
			END IF;
			IF delta > 0 THEN
				UPDATE users SET balance=balance-delta*fee_perday WHERE tariffid=tariff_id;
			END IF;
		END IF;
	END WHILE;
	CLOSE cur_fee_services;
END ;;

DROP PROCEDURE IF EXISTS `writeoff_month` ;;
CREATE PROCEDURE `writeoff_month`()
BEGIN
	DECLARE tariff_id, service_id, delta, b INT;
	DECLARE tariff_permonth, fee_permonth DOUBLE;
	DECLARE tariff_last_writeoff_month, fee_last_writeoff_month TIMESTAMP;
	DECLARE cur_t CURSOR FOR SELECT id, permonth, last_writeoff_month FROM tariffs;
	DECLARE cur_fee_services CURSOR FOR SELECT tariffid, serviceid, permonth, last_writeoff_month FROM fee_by_services;
	DECLARE CONTINUE HANDLER FOR NOT FOUND
		SET b=1;
	OPEN cur_t;
	WHILE b != 1 DO
		FETCH cur_t INTO tariff_id, tariff_permonth, tariff_last_writeoff_month;
		IF NOT b <=> 1 THEN
			IF tariff_last_writeoff_month = '0000-00-00 00:00:00' THEN
				SET delta = 1;
			ELSE
				SET delta = PERIOD_DIFF(DATE_FORMAT(now(),'%Y%m'), DATE_FORMAT(tariff_last_writeoff_month,'%Y%m'));
			END IF;
			
			IF delta > 0 THEN
				UPDATE users SET balance=balance-delta*tariff_permonth WHERE tariffid=tariff_id;
				UPDATE tariffs SET last_writeoff_month=now() WHERE id=tariff_id;
			END IF;
		END IF;
	END WHILE;
	CLOSE cur_t;
-- Writeoff services fee
	SET b = 0;
	OPEN cur_fee_services;
	WHILE NOT b <=> 1 DO
		FETCH cur_fee_services INTO tariff_id, service_id, fee_permonth, fee_last_writeoff_month;
		IF NOT b <=> 1 THEN
			IF fee_last_writeoff_month = '0000-00-00 00:00:00' THEN
				SET delta = 1;
			ELSE
				SET delta = PERIOD_DIFF(DATE_FORMAT(now(),'%Y%m'), DATE_FORMAT(fee_last_writeoff_month,'%Y%m'));
			END IF;
			IF delta > 0 THEN
				UPDATE users SET balance=balance-delta*fee_permonth WHERE tariffid=tariff_id;
			END IF;
		END IF;
	END WHILE;
	CLOSE cur_fee_services;
END ;;


DELIMITER ;

